/*!\file:  SpawnCoreParallel.cpp
 * \brief: run core ISSM solution using Dakota inputs coming from CPU 0.
 * \sa qmu.cpp DakotaPlugin.cpp
 *
 * This routine needs to be understood simultaneously with qmu.cpp and DakotaPlugin. 
 * SpawnCoreParallel is called by all CPUS, with CPU 0 holding Dakota variable values, along 
 * with variable descriptors. 
 *
 * SpawnCoreParallel takes care of broadcasting the variables and their descriptors across the MPI 
 * ring. Once this is done, we use the variables to modify the inputs for the solution core. 
 * For ex, if "rho_ice" is provided, for ex 920, we include "rho_ice" in the inputs, then 
 * call the core with the modified inputs. This is the way we get Dakota to explore the parameter 
 * spce of the core. 
 *
 * Once the core is called, we process the results of the core, and using the processed results, 
 * we compute response functions. The responses are computed on all CPUS, but they are targeted 
 * for CPU 0, which will get these values back to the Dakota engine. 
 *
 */ 

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif


#include "../../objects/objects.h"
#include "../../io/io.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "./Qmux.h"
#include "../../include/include.h"
#include "../../solutions/solutions.h"
#include "../modules.h"

void SpawnCoreParallel(double* responses, int numresponses, double* variables, char** variables_descriptors,int numvariables, FemModel* femmodel,int counter){

	int i;
	extern int my_rank;
	
	char   **responses_descriptors     = NULL;
	char    *string                    = NULL;
	int      string_length;
	int      verbose                   = 0;
	int      dummy;
	int      solution_type;
	void (*solutioncore)(FemModel*)    = NULL;

	/*synchronize all cpus, as CPU 0 is probably late (it is starting the entire dakota strategy!) : */
	MPI_Barrier(MPI_COMM_WORLD);
	_printf_("qmu iteration: %i\n",counter);
	
	/*retrieve parameters: */
	femmodel->parameters->FindParam(&verbose,VerboseEnum);
	femmodel->parameters->FindParam(&responses_descriptors,&dummy,ResponseDescriptorsEnum);
	femmodel->parameters->FindParam(&solution_type,SolutionTypeEnum);

	/* only cpu 0, running dakota is providing us with variables and variables_descriptors and numresponses: broadcast onto other cpus: */
	DakotaMPI_Bcast(&variables,&variables_descriptors,&numvariables,&numresponses);

	/*Modify core inputs in objects contained in femmodel, to reflect the dakota variables inputs: */
	InputUpdateFromDakotax(femmodel->elements,femmodel->nodes,femmodel->vertices,femmodel->loads,femmodel->materials,femmodel->parameters,femmodel->partition,variables,variables_descriptors,numvariables);

	/*Run the core solution sequence: */
	if(verbose)_printf_("%s%s%s\n","Starting ",EnumToString(solution_type)," core:");
	SolutionConfiguration(NULL,NULL,&solutioncore,solution_type); 
	solutioncore(femmodel);

	/*compute responses: */
	if(verbose)_printf_("compute dakota responses:\n");
	DakotaResponsesx(responses,femmodel->elements,femmodel->nodes,femmodel->vertices,femmodel->loads,femmodel->materials,femmodel->parameters,responses_descriptors,numresponses);

	/*Free ressources:*/
	DakotaFree(&variables,&variables_descriptors,&responses_descriptors, numvariables, numresponses);
}

